@@ -2,7 +2,7 @@ package MogileFS::Server;
 use strict;
 use warnings;
 use vars qw($VERSION);
-$VERSION = "2.55";
+$VERSION = "2.56";
 
 =head1 NAME
 
@@ -15,25 +15,6 @@ MogileFS::Server - MogileFS (distributed filesystem) server
 
 =cut
 
-# based on where we found this file (a pure-perl module),
-# add the mogdeps/ subdirectory of that base to our @INC search
-# path, where all the misc Mogile dependencies are installed.
-BEGIN {
-    my $libpath;
-    if (! $ENV{MOGILE_NO_BUILTIN_DEPS} &&
-        ($libpath = $INC{"MogileFS/Server.pm"}) &&
-        $libpath =~ s!MogileFS/Server.pm$!!)
-    {
-        my $dep_dir = "${libpath}mogdeps";
-        push @INC, $dep_dir;
-        unless (($ENV{PERL5LIB} || "") =~ /$dep_dir/) {
-            $ENV{PERL5LIB} = join(":",
-                                  split(/:/, $ENV{PERL5LIB} || ""),
-                                  $dep_dir);
-        }
-    }
-}
-
 use IO::Socket;
 use Symbol;
 use POSIX;
@@ -227,7 +208,16 @@ sub server {
 }
 
 # database checking/connecting
-sub validate_dbh { Mgd::get_store()->recheck_dbh }
+sub validate_dbh { 
+    my $sto = Mgd::get_store();
+    my $had_dbh = $sto->have_dbh;
+    $sto->recheck_dbh();
+    my $dbh;
+    eval { $dbh = $sto->dbh };
+    # Doesn't matter what the failure was; workers should retry later.
+    error("Error validating master DB: $@") if $@ && $had_dbh;
+    return $dbh;
+}
 sub get_dbh      { return Mgd::get_store()->dbh  }
 
 # the eventual replacement for callers asking for a dbh directly:
